/*
 * Copyright 2009 Red Hat, Inc.
 * Red Hat licenses this file to you under the Apache License, version
 * 2.0 (the "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.  See the License for the specific language governing
 * permissions and limitations under the License.
 */
package org.hornetq.core.example;

import java.util.Date;
import java.util.HashMap;

import org.hornetq.api.core.SimpleString;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.core.client.*;
import org.hornetq.core.config.Configuration;
import org.hornetq.core.config.impl.ConfigurationImpl;
import org.hornetq.core.remoting.impl.invm.InVMAcceptorFactory;
import org.hornetq.core.remoting.impl.invm.InVMConnectorFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.core.server.HornetQServers;
import org.hornetq.core.settings.impl.AddressSettings;

/**
 * 
 * This exammple shows how to run a HornetQ core client and server embedded in your
 * own application
 *
 * @author <a href="mailto:tim.fox@jboss.com">Tim Fox</a>
 *
 */
public class EmbeddedExample
{

   public static void main(final String[] args)
   {
      try
      {

         // Step 1. Create the Configuration, and set the properties accordingly
         Configuration configuration = new ConfigurationImpl();
         configuration.setPersistenceEnabled(false);
         configuration.setSecurityEnabled(false);
         configuration.getAcceptorConfigurations().add(new TransportConfiguration(InVMAcceptorFactory.class.getName()));
         
         // Step 2. Create and start the server
         HornetQServer server = HornetQServers.newHornetQServer(configuration);


         AddressSettings settings = new AddressSettings();
         settings.setDeadLetterAddress(new SimpleString("DLA"));
         settings.setMaxDeliveryAttempts(5);
         server.getAddressSettingsRepository().addMatch("#",settings);
         
         server.start();
         
         // Step 3. As we are not using a JNDI environment we instantiate the objects directly
         ClientSessionFactory sf = HornetQClient.createClientSessionFactory(new TransportConfiguration(InVMConnectorFactory.class.getName()));

         // Step 4. Create a core queue
         ClientSession coreSession = sf.createSession(false, false, false);

         final String queueName = "exampleQueue";
         
         coreSession.createQueue("DLA","DLQ",true);
         coreSession.createQueue(queueName, queueName, true);

         coreSession.close();

         ClientSession session = null;

         try
         {

            // Step 5. Create the session, and producer
            session = sf.createSession(false, false, false);

            ClientProducer producer = session.createProducer(queueName);

            // Step 6. Create and send a message
            ClientMessage message = session.createMessage(false);

            final String propName = "myprop";

            message.putStringProperty(propName, "Hello sent at " + new Date());

            System.out.println("Sending the message.");

            producer.send(message);
            
            session.commit();

            // Step 7. Create the message consumer and start the connection
            session.start();

            ClientConsumer messageConsumer = session.createConsumer(queueName);

            // Step 8. Receive the message.
            for (int i = 0 ; i < 100; i++)
            {
               ClientMessage messageReceived = messageConsumer.receive(1000);
               if (messageReceived == null)
               {
                  break;
               }
               messageReceived.acknowledge();
               System.out.println("Received a message " + messageReceived);
               session.rollback();
            }
            
            messageConsumer.close();
            
            
            messageConsumer = session.createConsumer(new SimpleString("DLQ"));
            
            ClientMessage messageReceived = messageConsumer.receive(1000);
            if (messageReceived == null)
            {
               throw new NullPointerException("Didn't receive a message");
            }
            
            System.out.println("Message received from DLQ");
            messageReceived.acknowledge();
            
            session.commit();

         }
         finally
         {
            // Step 9. Be sure to close our resources!
            if (session != null)
            {
               session.close();
            }

            // Step 10. Stop the server
            server.stop();
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();
         System.exit(-1);
      }
   }

   {
      HashMap<String, Object> map = new HashMap<String, Object>();
      map.put("host", "localhost");
      map.put("port", 5445);
      TransportConfiguration config = new TransportConfiguration(InVMConnectorFactory.class.getName(), map);
      ClientSessionFactory sf = HornetQClient.createClientSessionFactory(config);
   }

}
